;----------------------------------------------------------------------------
;     MODULE NAME:   NESTCHK.H
;
;         $Author:   Dennis_Bareis  $
;       $Revision:   1.1  $
;           $Date:   20 Apr 1999 17:43:10  $
;        $Logfile:   E:/DB/PVCS.IT/OS2/PPWIZARD/NESTCHK.H_V  $
;
;     DESCRIPTION:   This is a header file for generic handling of
;                    nesting checking. This is where you must have matching
;                    "whatevers" such as a matching "</TABLE>" for a
;                    "<TABLE>" etc.
;
;                    "TABLE" checking is a good example as are "FRAMESET"
;                    statements as the Netscape browser will not display
;                    an incorrectly nested page (common cause of "blank"
;                    pages). This header file can be used to detect these
;                    errors while still developing the pages rather than
;                    HOPING that testing will show up the errors.
;
;                    Because these macros are generic you would usually have
;                    your own header file to front end them, for example your
;                    header could begin with:
;
;                       ;--- Prepare nesting checking code ---
;                       #include "NESTCHK.H"
;                       <$NestingInit Id="TABLE" DESC="HTML TABLE tag">
;                       #define TABLE  <$NestingInc Id="TABLE">TABLE
;                       #define /TABLE <$NestingDec Id="TABLE"></TABLE>
;
;                    Your code would then always use the table macros defined
;                    above rather than the HTML table tags directly....
;
;                    You would probably do more that just "TABLE" tags in the
;                    above example! Note that I have made the "/TABLE" macro
;                    decrement the nesting level and output the HTML "/TABLE"
;                    tag as well.
;
;
;                    Macro "NestingInit"
;                    ~~~~~~~~~~~~~~~~~~~
;
;                    This macro takes the following parameters:
;
;                         ID
;                         ~~
;                         Manditory. Short name to describe what we wish
;                         to check.  Must contain only characters which
;                         are valid in rexx variable names (no checking of
;                         this is done).
;
;                         DESC
;                         ~~~~
;                         A description which is displayed if a nesting
;                         error is found.
;
;
;                    Macro "NestingInc"
;                    ~~~~~~~~~~~~~~~~~~
;
;                    This macro takes the following parameters:
;
;                         ID
;                         ~~
;                         Manditory. Matches that used on "NestingInit".
;                         You are increasing the level of nesting.
;
;
;                    Macro "NestingDec"
;                    ~~~~~~~~~~~~~~~~~~
;
;                    This macro takes the following parameters:
;
;                         ID
;                         ~~
;                         Manditory. Matches that used on "NestingInit".
;                         You are decreasing the level of nesting.
;
;
;                    Macro "NestingLevel"
;                    ~~~~~~~~~~~~~~~~~~~
;
;                    This macro takes the following parameters:
;
;                         ID
;                         ~~
;                         Manditory. Matches that used on "NestingInit".
;                         Returns the name of the variable which contains
;                         the level (0= neutral, 1 = down one etc).
;
;----------------------------------------------------------------------------

;--- Only include this header once! -----------------------------------------
#ifndef   VERSION_NESTCHK_H


;--- Define the version number of this header file --------------------------
#define   VERSION_NESTCHK_H    99.114
#require  99.112

;--- Register checking macro ------------------------------------------------
#OnExit  <$NestingCheck>

;--- Currently not tracking nesting of anything -----------------------------
#RexxVar NestingIdCount = 0

;--- Define macro to keep track of items ------------------------------------
#define NestingInit_REXX                                            \
        ^                                                           \
           ;--- Remember ID details --------------------------      \
           NestingIdCount = NestingIdCount + 1;                     \
           call value "NestingId.NestingIdCount.Id",    '{$ID}';    \
           call value "NestingId.NestingIdCount.Desc",  '{$DESC}';  \
                                                                    \
           ;--- Initialize level counter ---------------------      \
           call value "{$ID}_Level", 0;                             \
        ^
#define NestingInit                                                 \
        #evaluate  '' <$NestingInit_REXX ID='{$ID}' DESC='{$DESC}'>

;--- We are going down one level of nesting ---------------------------------
;                                                     [SaveSourceAndLocnInfo] (Ignore this - this bit used to include section into doco)
#define NestingInc_REXX                                       \
        ^                                                     \
           ;--- Increase nesting level ---                    \
           call value "{$ID}_Level", {$ID}_Level + 1;         \
                                                              \
           ;--- Record details about current location ---     \
           call value "{$ID}_FilePosn.{$ID}_Level", GetInputFileNameAndLine();   \
           call value "{$ID}_CurrLine.{$ID}_Level", GetFileLineBeingProcessed(); \
        ^
#define NestingInc                                            \
        #evaluate  '' <$NestingInc_REXX ID='{$ID}'>
;                                                     [SaveSourceAndLocnInfo] (Ignore this - this bit used to include section into doco)


;--- We are going up one level of nesting -----------------------------------
#define NestingDec                                            \
        #evaluate '' ^call value "{$ID}_Level", {$ID}_Level - {$DecreaseBy="1"}^

;--- Allow user to access the counter variable ------------------------------
#define NestingLevel                                          \
        {$ID}_Level


;--- Check nesting of all ID's ----------------------------------------------
#define NestingCheck_REXX                                       \
        ^                                                       \
           ;--- Loop though all registered "nesting" items ---  \
           NestingError = 0;       ;;No errors found yet        \
           do CheckIndex = 1 to NestingIdCount;                 \
               ;--- Get basic information about item ---        \
              _ID    = NestingId.CheckIndex.Id;                 \
              _DESC  = NestingId.CheckIndex.Desc;               \
                                                                \
              _Level = value(_ID || "_Level");                  \
              if _Level <> 0 then;                              \
              do;                                               \
                 if NestingError = 0 then;                      \
                 do;                                            \
                    say '';                                     \
                    say 'NESTING ERRORS';                       \
                    say '~~~~~~~~~~~~~~';                       \
                 end;                                           \
                 NestingError = NestingError + 1;               \
                 if _Level < 0 then;                            \
                     say 'Have ' || abs(_Level) || ' too many end nesting tags on "' || _DESC || '"';  \
                 else;                                          \
                 do;                                            \
                     ;--- display nesting info ---              \
                     say 'Missing ' || _Level || ' end nesting tag(s) on "' || _DESC || '"';  \
                     do _ThisLevel = 1 to _Level;                \
                        say '  * ' || value(_ID || "_FilePosn._ThisLevel");  \
                        say '    ' || value(_ID || "_CurrLine._ThisLevel");  \
                     end;                                       \
                 end;                                           \
              end;                                              \
           end;                                                 \
        ^
#define NestingCheck                                            \
        #evaluate  '' <$NestingCheck_REXX>                      \
        #if [NestingError <> 0]                                 \
            #error ^NestingCheck(): Found <??NestingError> nesting error(s) as described above^ \
        #endif



;--- Only include this header once! -----------------------------------------
#endif
